#include "libssh2_num.h"

/* This ultra-basic arbitrary length integer manipulator is
 * meant for positive integers only and makes assumptions
 * which may be inappropriate for use outside of libssh2
 */

/* {{{ libssh2_num_init
 * Allocate a number structure
 */
LIBSSH2_NUM *libssh2_num_init(LIBSSH2_SESSION *session)
{
	LIBSSH2_NUM *num;

	num = LIBSSH2_ALLOC(session, sizeof(LIBSSH2_NUM));
	if (!num) {
		return NULL;
	}
	memset(num, 0, sizeof(LIBSSH2_NUM));
	num->session = session;

	return num;
}
/* }}} */

/* {{{ libssh2_num_bin
 * Populate an arbitrary length integer from binary data
 */
int libssh2_num_bin(LIBSSH2_NUM *num, unsigned char *binnum, int binnum_len)
{
	if (num->val) {
		LIBSSH2_FREE(num->session, num->val);
		num->val = NULL;
		num->val_len = 0;
	}

	num->val = LIBSSH2_ALLOC(num->session, binnum_len);
	if (!num->val) {
		return -1;
	}
	memcpy(num->val, binnum, binnum_len);
	num->val_len = binnum_len;

	return 0;
}
/* }}} */

/* {{{ libssh2_num_multiply
 * Mutiply two numbers together
 */
int libssh2_num_multiply(LIBSSH2_NUM *result, LIBSSH2_NUM *a, LIBSSH2_NUM *b)
{
	int lpa, lpb;

	if (result->val) {
		LIBSSH2_FREE(result->session, result->val);
	}

	result->val_len = a->val_len + b->val_len;
	result->val = LIBSSH2_ALLOS(result->session, result->val_len);
	if (!result->val) {
		result->val_len = 0;
		return -1;
	}
	for(lpa = 0; lpa < a->val_len; lpa++) {
		for(lpb = 0; lpb < b->val_len; lpb++) {
			int prod = a->val[lpa] * b->val[lpb], i;
			unsigned char msp = (prod >> 8) & 0xFF, lsp = prod & 0xFF;

			if (result->val[lpa + lpb + 1] + lsp > 255) {
				for(i = (lpa + lpb); i >= 0; i--) {
					result->val[i]++;
					if (result->val[i]) break;
				}
			}
			result->val[lpa + lpb + 1] += lsp;

			if (result->val[lpa + lpb] + msp > 255) {
				for(i = (lpa + lpb - 1); i >= 0; i--) {
					result->val[i]++;
					if (result->val[i]) break;
				}
			}
			result->val[lpa + lpb] += msp;
		}
	}

	return 0;
}
/* }}} */

/* {{{ libssh2_num_expmod
 * A^B mod C
 */
int libssh2_num_expmod(LIBSSH2_NUM *result, LIBSSH2_NUM *base, LIBSSH2_NUM *pow, LIBSSH2_NUM *mod)
{

}
/* }}} */
